__all__ = ["gitRun", "gitSave", "gitAdd", "gitRemove"]

import asyncio

from .configuration import cfg, PATH


async def gitRun(*args):
    if not cfg["database"]["use_git"]:
        return
    cmd_args = ["git", f"--git-dir={PATH}.git", f"--work-tree={PATH}"]
    cmd_args.extend(args)
    proc = await asyncio.create_subprocess_exec(*cmd_args)
    await proc.communicate()


async def gitSave(path, message="backup"):
    await gitRun("commit", path + ".md", "-m", message)


async def gitAdd(path):
    await gitRun("add", path + ".md")


async def gitRemove(path):
    await gitRun("rm", "-f", path + ".md")



from __future__ import annotations

__all__ = ["app"]

import os
from typing import List, Dict

import asyncio
import aiofiles

from fastapi import FastAPI, UploadFile, File, Depends
from fastapi.staticfiles import StaticFiles
from starlette.responses import RedirectResponse

from .models import Note
from .configuration import cfg, PATH, IMAGE_PATH, FRONT
from .versionning import gitRun, gitRemove, gitSave
from .authentication import get_current_user_from_token, init as auth_init

try:
    import orjson
except ImportError:
    app = FastAPI(debug=True)
else:
    from fastapi.responses import ORJSONResponse

    app = FastAPI(debug=True, default_response_class=ORJSONResponse)


@app.get("/")
def index():
    return RedirectResponse(url="/index.html")


@app.post("/upload")
async def upload(
    file: UploadFile = File(...),
    current_user: dict = Depends(get_current_user_from_token),
):
    fullname = os.path.join(IMAGE_PATH, file.filename)
    async with aiofiles.open(fullname, "wb") as f:
        await f.write(await file.read())


@app.delete("/notebook")
async def deleteNote(
    name: str, current_user: dict = Depends(get_current_user_from_token)
):
    """Remove one note"""
    await gitRemove(name)
    await gitSave(name, f"Removed {name}")


@app.post("/notebook")
async def addNote(
    note: Note, current_user: dict = Depends(get_current_user_from_token)
) -> Dict[str, str]:
    """Create one note"""
    assert "." not in note.name
    note.name = os.path.join(current_user["name"], note.name)
    assert not os.path.exists(note.filename)
    await note.save(creation=True)
    return dict(name=note.name)


@app.put("/notebook")
async def updateNote(
    note: Note, current_user: dict = Depends(get_current_user_from_token)
):
    """Update one note"""
    assert os.path.exists(note.filename)
    await note.save()


@app.get("/notebooks")
async def getNotes(
    current_user: dict = Depends(get_current_user_from_token),
) -> List[Note]:
    """Fetches the notebook"""
    entries = []
    for root, _dirs, files in os.walk(os.path.join(PATH, current_user["name"])):
        for file in files:
            if file.endswith(".md"):
                parent = root[len(PATH) :]
                doc = file[:-3]
                entries.append(await Note.load(os.path.join(parent, doc)))
    return entries


auth_init(app)
app.mount("/images/", StaticFiles(directory=IMAGE_PATH), name="images")
app.mount("/", StaticFiles(directory=FRONT), name="static")

if cfg["database"]["use_git"]:
    if not os.path.exists(os.path.join(PATH, ".git")):
        asyncio.gather(gitRun("init"))

    fullpath = os.path.abspath(os.path.expanduser(os.path.expandvars(PATH)))
    cwd = os.getcwd()
    os.chdir(PATH)
    for root, _dirs, files in os.walk(fullpath):
        for fname in files:
            if fname.endswith(".md"):
                os.system(f'git add "{root[len(fullpath)+1:]}/{fname}"')
    os.system('git commit -m "Wiki startup"')
    os.chdir(cwd)



#!/bin/env python
import os
import sys
import shutil

PORT = 8000
HOST = "127.0.0.1"
SIMPLE = os.environ.get('FG', False)

if len(sys.argv) > 1:
    if sys.argv[1] == "new":
        if len(sys.argv) > 2:
            from wirikiki.configuration import ROOT

            dst = sys.argv[2]
            shutil.copytree(os.path.join(ROOT, "config"), dst)
            for path in ("web", "myKB"):
                shutil.copytree(os.path.join(ROOT, path), os.path.join(dst, path))

            settings = os.path.join(dst, "settings.toml")
            data = open(settings).read()
            with open(settings, "w") as f:
                for line in data.split("\n"):
                    if line.startswith("base_dir"):
                        f.write("base_dir = %r\n" % os.path.join(os.getcwd(), dst))
                    else:
                        f.write(line + "\n")

            raise SystemExit(0)
        else:
            print("Syntax: %s %s <name>" % (os.path.basename(sys.argv[0]), sys.argv[1]))
            raise SystemExit(1)
    elif sys.argv[1] == "update":
        from wirikiki.configuration import ROOT
        shutil.rmtree("web")
        shutil.copytree(os.path.join(ROOT, "web"), "web")
        raise SystemExit(0)
    if sys.argv[1] == "help":
        print("""Possible options:

* new <name>
* update

Without arguments it will just run the wiki in the current folder.""")
        raise SystemExit(0)


def run():
    import os
    from wirikiki.configuration import USING_DEFAULTS

    if USING_DEFAULTS:
        print(
            "Can't find settings.toml in the current folder, set CFG_FILE in the environment and try again"
        )
        raise SystemExit(-1)

    pid = 0 if SIMPLE else os.fork()

    if pid > 0:  # main process, launch browser
        import time
        import webbrowser

        time.sleep(1)
        webbrowser.open(f"http://{HOST}:{PORT}")
    else:  # daemon (children) process
        import sys
        import uvicorn

        try:
            from setproctitle import setproctitle
        except ImportError:
            pass
        else:
            setproctitle(sys.argv[0])
        if not SIMPLE:
            os.setsid()  # detach
        uvicorn.run("wirikiki.routes:app", host=HOST, port=PORT, log_level="warning")


if __name__ == "__main__":
    run()



__all__ = ["cfg", "IMAGE_PATH", "PATH", "USERS"]

import os
from string import Template
from typing import Dict, Any

import wirikiki

import tomli

config_fname = os.environ.get("CFG_FILE", os.path.join(os.path.curdir, "settings.toml"))
USING_DEFAULTS = False

if not os.path.exists(config_fname):
    config_fname = os.path.join(os.path.dirname(wirikiki.__file__), 'config', 'settings.toml')
    USING_DEFAULTS = True

cfg: Dict[str, Dict[str, Any]] = tomli.load(open(config_fname, "rb"))

ROOT = os.path.dirname(wirikiki.__file__)
PATH = os.environ.get(
    "DBDIR",
    Template(cfg["database"]["directory"]).substitute(root=cfg["general"]["base_dir"]),
)
USERS = Template(cfg["users"]["database"]).substitute(root=cfg["general"]["base_dir"])

FRONT = os.path.join(ROOT, "web")


if not os.path.exists(PATH):
    PATH = os.path.curdir

if not PATH.endswith(os.path.sep):
    PATH += os.path.sep

IMAGE_PATH = os.path.join(PATH, "images")

if not os.path.exists(IMAGE_PATH):
    os.mkdir(IMAGE_PATH)



#!/bin/env python
import os
import sys
from wirikiki.authentication import make_password


def run():
    if len(sys.argv) == 1:
        print(
            "To generate a hash suitable for the user database, provide the clear password"
        )
        print("Syntax: %s <password>" % (os.path.basename(sys.argv[0])))
        raise SystemExit(-1)
    else:
        print(make_password(sys.argv[1]))


if __name__ == "__main__":
    run()



import hashlib

from typing import Optional, Dict
from fastapi import HTTPException, status, Depends
from pydantic import BaseModel
from fastapi.security import OAuth2PasswordRequestForm, OAuth2PasswordBearer
from datetime import timedelta, datetime
from jose import JWTError, jwt  # type: ignore

from .configuration import cfg, USERS


def create_access_token(data: dict, expires_delta: Optional[timedelta] = None):
    data_copy = data.copy()
    expire = datetime.utcnow() + (
        expires_delta
        if expires_delta
        else timedelta(minutes=cfg["token"]["expire_minutes"])
    )
    data_copy.update({"exp": expire})
    return jwt.encode(
        data_copy, cfg["token"]["key"], algorithm=cfg["token"]["algorithm"]
    )


_user_db: Dict[str, str] = {}


def get_user_db() -> Dict[str, str]:
    if not _user_db:
        for line in open(USERS, "r", encoding="utf-8"):
            user, password = line.split(" ", 1)
            _user_db[user] = password.strip()
    return _user_db


class Token(BaseModel):
    access_token: str
    token_type: str


credentials_exception = HTTPException(
    status_code=status.HTTP_401_UNAUTHORIZED,
    detail="Could not validate credentials",
)


oauth2_scheme = OAuth2PasswordBearer(tokenUrl="token")


def get_current_user_from_token(
    token: str = Depends(oauth2_scheme), db: dict = Depends(get_user_db)
) -> Dict[str, str]:
    try:
        payload = jwt.decode(
            token, cfg["token"]["key"], algorithms=[cfg["token"]["algorithm"]]
        )
        username: str = payload.get("sub")
        if username is None:
            raise credentials_exception
    except JWTError:
        raise credentials_exception

    if not (cfg["users"]["allow_anonymous"] and username == "anonymous"):
        user = db.get(username)
        if user is None:
            raise credentials_exception
    return dict(name=username)


def make_password(password: str) -> str:
    text = (cfg["users"]["salt"] + password).encode("utf-8")
    return hashlib.sha512(text).hexdigest()


def init(application):
    @application.post("/token", response_model=Token)
    def _login_for_access_token(
        form_data: OAuth2PasswordRequestForm = Depends(),
        db: dict = Depends(get_user_db),
    ) -> Dict[str, str]:
        is_anonymous = (
            form_data.username == "anonymous" and cfg["users"]["allow_anonymous"]
        )
        if is_anonymous or (
            make_password(form_data.password) == db.get(form_data.username)
        ):
            access_token = create_access_token(
                data={"sub": form_data.username},
            )
            return {"access_token": access_token, "token_type": "bearer"}
        else:
            raise credentials_exception



from __future__ import annotations

__all__ = ["Note"]

import os
from pydantic import BaseModel

import aiofiles

from .versionning import gitAdd, gitSave
from .configuration import PATH


class Note(BaseModel):
    "A markdown note"

    name: str
    content: str = ""

    @property
    def exists(self) -> bool:
        return os.path.exists(self.filename)

    @property
    def filename(self) -> str:
        return os.path.join(PATH, self.name) + ".md"

    @staticmethod
    async def load(name) -> Note:
        n = Note(name=name)
        async with aiofiles.open(n.filename, "r", encoding="utf-8") as f:
            n.content = await f.read()
        return n

    async def save(self, creation=False):
        if creation:  # auto make directory if it doesn't exist
            rootDir = os.path.dirname(self.filename)
            if not os.path.exists(rootDir):
                os.makedirs(rootDir, exist_ok=True)
        async with aiofiles.open(self.filename, "w", encoding="utf-8") as f:
            await f.write(self.content)
        if creation:
            await gitAdd(self.name)
        await gitSave(
            self.name,
            f"Udated {self.name}" if not creation else f"NEW note: {self.name}",
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['wirikiki', 'wirikiki.cli']

package_data = \
{'': ['*'],
 'wirikiki': ['config/*',
              'myKB/.git/*',
              'myKB/.git/hooks/*',
              'myKB/.git/info/*',
              'myKB/.git/logs/*',
              'myKB/.git/logs/refs/heads/*',
              'myKB/.git/objects/02/*',
              'myKB/.git/objects/07/*',
              'myKB/.git/objects/16/*',
              'myKB/.git/objects/23/*',
              'myKB/.git/objects/25/*',
              'myKB/.git/objects/35/*',
              'myKB/.git/objects/36/*',
              'myKB/.git/objects/38/*',
              'myKB/.git/objects/42/*',
              'myKB/.git/objects/5b/*',
              'myKB/.git/objects/5d/*',
              'myKB/.git/objects/78/*',
              'myKB/.git/objects/8a/*',
              'myKB/.git/objects/91/*',
              'myKB/.git/objects/94/*',
              'myKB/.git/objects/9a/*',
              'myKB/.git/objects/af/*',
              'myKB/.git/objects/b4/*',
              'myKB/.git/objects/c5/*',
              'myKB/.git/objects/cb/*',
              'myKB/.git/objects/d1/*',
              'myKB/.git/objects/d3/*',
              'myKB/.git/objects/d5/*',
              'myKB/.git/objects/de/*',
              'myKB/.git/objects/e1/*',
              'myKB/.git/objects/e3/*',
              'myKB/.git/objects/ec/*',
              'myKB/.git/refs/heads/*',
              'myKB/anonymous/*',
              'myKB/fab/*',
              'service/*',
              'web/*',
              'web/css/*',
              'web/css/imgs_dhx_material/*',
              'web/img/*',
              'web/js/*']}

install_requires = \
['aiofiles>=0.8.0,<0.9.0',
 'fastapi>=0.79.0,<0.80.0',
 'orjson>=3.7.11,<4.0.0',
 'python-jose>=3.3.0,<4.0.0',
 'python-multipart>=0.0.5,<0.0.6',
 'setproctitle>=1.3.0,<2.0.0',
 'tomli>=2.0.1,<3.0.0',
 'uvicorn>=0.18.2,<0.19.0']

entry_points = \
{'console_scripts': ['wirikiki = wirikiki.cli.wirikiki:run',
                     'wirikiki-pwgen = wirikiki.cli.wirikiki_pwgen:run']}

setup_kwargs = {
    'name': 'wirikiki',
    'version': '1.1.2',
    'description': 'A tiny desktop wiki',
    'long_description': '_Wirikiki_ aims at being a productivity oriented **markdown notebook**.\nIt\'s compatible with the most common features supported by GitHub\'s markdown and adds a few other.\n\nAims at\n\n- fast / **small** footprint\n- **simplicity** of use\n- **uncluttered** UI supporting both **dark & bright modes**\n- **advanced markdown support** via [markItDown](https://markitdown.netlify.app/) and plugins and [Marked](https://marked.js.org/) for the editor\n- auto **archival** (using git)\n\n# Installation\n\n## Building\n\n3 ways to get a runnable code:\n\n- install a release (eg: using `pip`)\n- `make dev`\n- manual installation\n\n```\nsh ./makevueApps.sh\nnpm install\nDIST=1 ./node_modules/.bin/rollup -c rollup.config.js\n```\n\n# Usage\n\n## Create one wiki\n\n```\nwirikiki new myNewWiki\n```\n\n## Run the wiki\n\nGo to the newly created folder "myNewWiki" and run:\n\n```\nwirikiki\n```\n\n## Zero-install mode\n\nYou can just open the html file to get a degraded experience, data will not be saved.\n\n## Keyboard shortcuts\n\n| Key            | Description                                     |\n| -------------- | ----------------------------------------------- |\n| **Escape**     | Close editor or modal, else toggles the sidebar |\n| **Del**        | **D**elete current page                         |\n| **E**          | **E**dit current note                           |\n| **F**          | Search/**F**ind something                       |\n| **N**          | **C**reate a new note                           |\n| **Left/Right** | Switch to previous/next note                    |\n\n# Advanced usage\n\nBasic git support is provided, to enable it just type `git init` in the `myKB` folder.\n\n```shell\ncd myKB\ngit init\n```\n\n# Dependencies\n\n- **Python 3**\n  - if you want to run it **without the virtualenv** you will need the following python packages:\n    - aiofiles\n    - fastapi\n    - uvicorn\n- nodejs and npm (BUILD ONLY)\n- inotify-tools (DEV ONLY)\n\n# Developers note\n\nThis is built using\n[Vue.js version 3](https://v3.vuejs.org/) for the front\nand [FastAPI](https://fastapi.tiangolo.com/) for the server side.\n',
    'author': 'fdev31',
    'author_email': 'fdev31@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'http://github.com/fdev31/wirikiki/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
